
% Reed-Solomon Encoder test
%----------------------------------
% references:
% [1] Candidate Standard: ATSC-M/H Standard, Part 2 - RF/Transmission
% System Caracteristics (A/153, Part 2: 2009) 
% 25 November 2008
% RS encoder - decoder as specified in section 5.4.9
% the RS parameters can be adjusted to fit any other RS code
% contained in standard 

disp(' ');
disp(' ');
disp('Setting up RS code and GF parameters');


RScode.n = 207;     % codeword length
RScode.k = 187;     % data length
RScode.t = (RScode.n-RScode.k)/2; % number of errors to be corrected
RScode.gen_poly = [1 152 185 240 5 111 99 6 220 112 150 69 36 187 22 228 198 121 121 165 174]; % generator polynomial (gf(256))
if (exist('gfparams.mat')== 2)
    load gfparams
    if and(gfpar.m == 8, gfpar.prim_poly == 285)
        RScode.gf = gfpar;
    else
        RScode.gf.m = 8;    % galois field GF(2^8) 
        RScode.gf.prim_poly = 2^8+2^4+2^3+2^2+2^0; % GF primitive polynomial
        RScode.gf.tables = tables; % initialization of gf structure 
    end;
else
    RScode.gf.m = 8;    % galois field GF(2^8) 
    RScode.gf.prim_poly = 2^8+2^4+2^3+2^2+2^0; % GF primitive polynomial
    RScode.gf.tables = tables; % initialization of gf structure 
end;

RScode.group_format = make_group_format('annex_a.dat'); % creates group format structure table
RScode.tp = 0;  % starting TP (see annex A)
RScode.tpmax = 117;

gfpar = RScode.gf;

save gfparams gfpar;


% number of testvectors
test_messages = 100;

disp('Generating testvectors');
message = round(rand(test_messages,RScode.k).* (2^RScode.gf.m-1));
coded_message   = [message zeros(test_messages, 2*RScode.t)];

disp(['Encoding ' num2str(test_messages) ' test message(s)']);
disp('Encoding progress:');
progress = 0;
disp([num2str(progress) '%']);
for ii = 1:test_messages
    % 
    % encode row
    %
    coded_message(ii,:) = RSencode(message(ii,:),RScode);
    
    progtmp = round(ii/test_messages*100);
    if (progress ~= progtmp)
        progress = progtmp;
        disp([num2str(progress) '%']);
    end;
end;

% non systematic encoding
disp('Non-systematic encoding ... ');
NS_coded_message = zeros(size(coded_message));
for i = 1:test_messages
    NS_coded_message(i,:) = NSencode(coded_message(i,:),RScode);
    if (RScode.tp == RScode.tpmax)
        RScode.tp = 0;
    else
        RScode.tp = RScode.tp + 1;
    end;
end;

decoded_message = message.*0;
disp('Injecting errors ... ')
resOK = 1;
max_errors = RScode.t+1;
progress = 0;
for errors = 0:max_errors;
    disp(['Number of simulated errors: ' num2str(errors) ' of ' num2str(max_errors)]);
    err_NS_coded_message = zeros(test_messages,RScode.n);
    for it = 1:test_messages
        err_NS_coded_message(it,:) = gfadd(NS_coded_message(it,:),make_error_vector(RScode.n,errors,RScode.gf),RScode.gf);
    end;
    
    disp('Non-systematic decoding ...');
    err_coded_message = zeros(test_messages,RScode.n);
    RScode.tp = 0;
    for it = 1:test_messages
        err_coded_message(it,:) = NSdecode(err_NS_coded_message(it,:),RScode);
        if (RScode.tp == RScode.tpmax)
            RScode.tp = 0;
        else
            RScode.tp = RScode.tp + 1;
        end;
    end;
    
    disp(['Decoding ' num2str(test_messages) ' test message(s)']);
    disp('Decoding progress:');

    disp([num2str(progress) '%']);
    f = 0;
    for ii = 1:test_messages
        % 
        % decode row
        %
        [decoded_message(ii,:), fail] = RSdecode(err_coded_message(ii,:),RScode);
        f = or(f,fail);
        progtmp = round((ii+errors*test_messages)/(test_messages*(max_errors+1))*100);
        if (progress ~= progtmp)
            progress = progtmp;
            disp([num2str(progress) '%']);
        end;
    end;
    if and(sum(sum(decoded_message - message)) ~= 0,f == 0)
        resOK = 0;
        break;
    end;
    
    % check fail flag for t+1 errors
    if and(errors == max_errors, f == 0)
        resOK = 0;
    end;
        
    % the result for t+1 errors must be data contained in codeword
    if and(and(errors == max_errors, f == 1), sum(sum(decoded_message - err_coded_message(:,1:RScode.k))) ~= 0)
        resOK = 0;
    end;
end;

if (resOK == 1)    
    disp('All results OK');
else
    disp('!!! Error in results !!!');
end;
    





